/*global Drupal, jQuery*/
Drupal.behaviors.euccRocketship = {
  attach: function (context, settings) {
    (function ($) {

      // Init when the popup has opened
      var dynamicButtonChanged = false;
      var body = $('body');
      body.once('euccRocketshipOpen').on('eu_cookie_compliance_popup_open', function() {
        $('.eu-cookie-compliance-banner-info').once('euccRocketship').each(function () {
          initCloseEvent();
          initStateLabel();
          initDynamicButton();
          initAcceptMinimal();
        });
        if (typeof settings.eu_cookie_compliance_rocketship__excluded_path == 'undefined' || !settings.eu_cookie_compliance_rocketship__excluded_path) {
          body.addClass('eucc-popup-showing');
          // Initiate accessibility for the modal.
          initAccessibility();
        }
      });
      body.once('euccRocketshipClose').on('eu_cookie_compliance_rocketship__popup_close', function() {
        body.removeClass('eucc-popup-showing');
      });
      body.once('euccRocketshipOpenExcluded').on('eu_cookie_compliance_rocketship__show_excluded', function() {
        body.addClass('eucc-popup-showing');
      });

      /**
       * Helper function to trigger an event when the popup is closed
       */
      function initCloseEvent() {
        var buttons = $('.eu-cookie-compliance-banner-info button');
        buttons.once('euccRocketshipCloseEvent').click(function (e) {
          $(this).trigger('eu_cookie_compliance_rocketship__popup_close')
          Drupal.attachBehaviors('#sliding-popup',settings);
        });
      }

      /**
       * Helper function to initialize the state label functionality
       */
      function initStateLabel() {
        var categories = $('.eu-cookie-compliance-category');
        categories.each(function () {
          var category = $(this);
          var check = category.find('input[type="checkbox"]');
          var stateLabel = category.find('.state-label');
          var aria = category.find('.toggle');
          check.change(function () {
            if (check.prop('checked')) {
              stateLabel.addClass('visible');
              aria.attr('aria-checked','true')
            }
            else {
              stateLabel.removeClass('visible');
              aria.attr('aria-checked','false')
            }
          });
        });
      }

      /**
       * Helper function to initialize the dynamic button
       */
      function initDynamicButton() {
        var dynamicButton = $('.eu-cookie-compliance-rocketship--dynamic-accept');
        updateDynamicButtonText(dynamicButton, true);
        var categoryChecks = $('.eu-cookie-compliance-category input[type="checkbox"]');
        categoryChecks.change(function() {
          updateDynamicButtonText(dynamicButton, false);
        });

        dynamicButton.click(function (e) {
          e.preventDefault();
          if (dynamicButton.hasClass('all')) {
            $('.agree-button.eu-cookie-compliance-default-button').click();
          }
          else {
            $('.eu-cookie-compliance-save-preferences-button').click();
          }
        });
      }

      /**
       * Helper function to update the dynamic button text
       */
      function updateDynamicButtonText(dynamicButton, useSaved) {
        var savedCategories = Drupal.eu_cookie_compliance.getAcceptedCategories();
        var selectedCategories = $('.eu-cookie-compliance-category:not(.disabled) input[type="checkbox"]:checked');
        if (selectedCategories.length || (useSaved && savedCategories.length)) {
          dynamicButton.html(dynamicButton.attr('data-selection'));
          dynamicButton.removeClass('all');
          dynamicButton.addClass('selected');
          dynamicButtonChanged = true;
        }
        else {
          if (!dynamicButtonChanged) {
            dynamicButton.html(dynamicButton.attr('data-all'));
            dynamicButton.removeClass('selected');
            dynamicButton.addClass('all');
          }
        }
      }

      /**
       * Helper function to initialize the accept minimal link
       */
      function initAcceptMinimal() {
        var acceptMinimalLink = $('.eu-cookie-compliance-rocketship--accept-minimal');
        var categoryChecks = $('.eu-cookie-compliance-category:not(.disabled) input[type="checkbox"]');
        acceptMinimalLink.click(function (e) {
          e.preventDefault();
          categoryChecks.prop('checked', false);
          $('.eu-cookie-compliance-save-preferences-button').click();
        });
      }

      /**
       * Helper function to initialize accessibility features.
       */
      function initAccessibility(){
        // Elements are not reachable via sequential keyboard navigation.
        let background = $('.page-wrapper, .toolbar');
        background.attr('aria-hidden',true);
        $('.skip-link').attr('tabindex','-1');
        $('.eu-cookie-compliance-rocketship--buttons a').on('click', function(e) {
          background.attr('aria-hidden','false');
          $('.skip-link').attr('tabindex','0');
          $('.page-wrapper a').first().focus();
        })

        // Prevent tabbing.
        $('.toolbar a, .page-wrapper a, img').on('keydown', function(e) {
          if($('body').hasClass('eucc-popup-showing')){
            if (e.keyCode === 9) e.preventDefault();
          }
        });

        // Go back to first focusable item of the modal.
        $('.eu-cookie-compliance-rocketship--dynamic-accept').on('keydown', function (e) {
          if ($("this:focus") && (e.which === 9) && (!e.shiftKey)) {
            e.preventDefault();
            $('.popup-content a, .toggle').first().focus();
          }
        });

        // Go to last focusable item of the modal.
        $('.popup-content a, .toggle').first().on('keydown',function (e) {
          if ($("this:focus") && (e.which === 9) && (e.shiftKey)) {
            e.preventDefault();
            $('.popup-content a, .toggle').last().focus();
          }
        })

        // Let users switch the categories with space.
        let categories = body.find(".eu-cookie-compliance-category");
        categories.each(function(e){
          let category = $(this);
          // When checkbox is not disabled, you can switch state
          if(!category.hasClass('disabled')) {
            let toggle = category.find('[role="checkbox"]');
            toggle.keyup(function (e){
              // When hit spacebar change state.
              if(e.which === 32){
                let input = category.find('input');
                // Toggle state.
                input.prop('checked',!input.prop("checked"));
                // Change dynamic button state.
                initDynamicButton();
              }
            })
          }
        });
      }
    }(jQuery));
  }
};
